module Gradient

export ∇₂!, ∇₂ᵀ!, ∇₃!, ∇₃ᵀ!, vec∇₃!, vec∇₃ᵀ!

function ∇₂!(u₁,u₂,u)
    @. @views begin
        u₁[1:(end-1), :] = u[2:end, :] - u[1:(end-1), :]
        u₁[end, :, :] = 0

        u₂[:, 1:(end-1)] = u[:, 2:end] - u[:, 1:(end-1)]
        u₂[:, end] = 0
    end
    return u₁, u₂
end

function ∇₂ᵀ!(v,v₁,v₂)
    @. @views begin
        v[2:(end-1), :] = v₁[1:(end-2), :] - v₁[2:(end-1), :]
        v[1, :] = -v₁[1, :]
        v[end, :] = v₁[end-1, :]

        v[:, 2:(end-1)] += v₂[:, 1:(end-2)] - v₂[:, 2:(end-1)]
        v[:, 1] += -v₂[:, 1]
        v[:, end] += v₂[:, end-1]
    end
    return v
end

function ∇₃!(u₁,u₂,u₃,u)
    @. @views begin
        u₁[1:(end-1), :, :] = u[2:end, :, :] - u[1:(end-1), :, :]
        u₁[end, :, :] = 0

        u₂[:, 1:(end-1), :] = u[:, 2:end, :] - u[:, 1:(end-1), :]
        u₂[:, end, :] = 0

        u₃[:, :, 1:(end-1)] = u[:, :, 2:end] - u[:, :, 1:(end-1)]
        u₃[:, :, end] = 0
    end
    return u₁, u₂, u₃
end

function ∇₃ᵀ!(v,v₁,v₂,v₃)
    @. @views begin
        v[2:(end-1), :, :] = v₁[1:(end-2), :, :] - v₁[2:(end-1), :, :]
        v[1, :, :] = -v₁[1, :, :]
        v[end, :, :] = v₁[end-1, :, :]

        v[:, 2:(end-1), :] += v₂[:, 1:(end-2), :] - v₂[:, 2:(end-1), :]
        v[:, 1, :] += -v₂[:, 1, :]
        v[:, end, :] += v₂[:, end-1, :]

        v[:, :, 2:(end-1)] += v₃[:, :, 1:(end-2)] - v₃[:, :, 2:(end-1)]
        v[:, :, 1] += -v₃[:, :, 1]
        v[:, :, end] += v₃[:, :, end-1]
    end
    return v
end

function vec∇₃!(u₁,u₂,u₃,u)
    @. @views for j=1:size(u, 1)
        ∇₃!(u₁[j, :, :, :],u₂[j, :, :, :],u₃[j, :, :, :],u[j, :, :, :])
    end
    return u₁, u₂, u₃
end

function vec∇₃ᵀ!(u,v₁,v₂,v₃)
    @. @views for j=1:size(u, 1)
        ∇₃ᵀ!(u[j, :, :, :],v₁[j, :, :, :],v₂[j, :, :, :],v₃[j, :, :, :])
    end
    return u
end

end # Module
