
module TestSymgrad

include("Util.jl")
include("Tensor.jl")
include("Gradient.jl")
using .Tensor
using .Gradient
using JLD

fn="test_symgrad_data.jld"
if isfile(fn)
    d=load(fn)
    x=d["x"]
    y=d["y"]
else
    x=randn(Float64, (6, 11, 12, 13))
    y=randn(Float64, (10, 11, 12, 13))
    save(fn, "x", x, "y", y)
end

s=[0.0, 0.0]

if true

    xtmp=similar(x)
    ytmp=similar(y)

    symmat₃ₓ₃ℰ!(ytmp, x)
    symmat₃ₓ₃ℰᵀ!(xtmp, y)

    ind = CartesianIndices(size(x)[2:end])

    for i in ind
        s[1]=s[1]+sum(ytmp[:, i].*weights_symten₃ₓ₃ₓ₃.*y[:, i])
        s[2]=s[2]+sum(xtmp[:, i].*weights_symmat₃ₓ₃.*x[:, i])
    end

elseif true

    x=x[1, :, :, :]
    y=y[1:3, :, :, :]

    xtmp=similar(x)
    ytmp=similar(y)

    ∇₃!(@view(ytmp[1, :, :, :]), @view(ytmp[2, :, :, :]), @view(ytmp[3, :, :, :]), x)
    ∇₃ᵀ!(xtmp, @view(y[1, :, :, :]), @view(y[2, :, :, :]), @view(y[3, :, :, :]))

    ind = CartesianIndices(size(x)[1:end])
    for i in ind
        s[1]=s[1]+sum(ytmp[:, i].*y[:, i])
        s[2]=s[2]+xtmp[i]*x[i]
    end
else

    x=x[1, :, :, 1]
    y=y[1:2, :, :, 1]

    xtmp=similar(x)
    ytmp=similar(y)

    ∇₂!(@view(ytmp[1, :, :]), @view(ytmp[2, :, :]), x)
    ∇₂ᵀ!(xtmp, @view(y[1, :, :]), @view(y[2, :, :]))

    ind = CartesianIndices(size(x)[1:end])
    for i in ind
        s[1]=s[1]+sum(ytmp[:, i].*y[:, i])
        s[2]=s[2]+xtmp[i]*x[i]
    end

end

print(s)

end
